unit PollinLCDBasics;

//gendert: 2002-03-12

interface


uses Sysutils, Dialogs;

const
  TEXT_ROWS  = 8;   //Textzeilen
  GRAPH_ROWS = 64;  //Graphikzeilem
  TEXT_HOME_ADDRESS  = $0000; (* Text Home @ 0000h *)
  TEXT_COLUMS        = 40;    (* Text Area Number (Colums) = 40 *)
  GRAPH_HOME_ADDRESS = $0200; (* Graph Home @ 0200h *)
  GRAPH_COLUMS       = 40;    (* Graph Area Number (Colums) = 40 *)
  CGRAM_HOME_ADDRESS = $1000; (* CG RAM Start ab Adresse 4096 = 1000h *)

  dm_or  = 0;
  dm_xor = 1;
  dm_and = 2;

function IntMinMax(I, Min, Max: Integer): Integer;

function StartLPTDriver: Boolean;

procedure WaitDisplayReady(Port: Integer; BiDi: Boolean);
procedure WriteData(Port: Integer; BiDi: Boolean; Data: Byte);
procedure WriteCtrl(Port: Integer; BiDi: Boolean; Command: Byte);
procedure InitLCD(Port: Integer; BiDi: Boolean);
procedure SetAddressPointer(Port: Integer; BiDi: Boolean; Addr: Integer);

procedure delay(ms: integer);

// Text vom Windows-Zeichensatz in den
// Display-Zeichensatz konvertieren, soweit mglich
// Displayzeichensatz hat einen Offset von 32
function ANSI2DisplayText(Chr: Integer): Integer;


implementation

uses PortIO;

var
 msdelay, delz: int64; // Bentigt, um Delay auszumessen
 t: TDateTime;
 LPTPort: TDLPortIO;

function IntMinMax(I, Min, Max: Integer): Integer;
begin
  Result := i;
  if i < Min then Result := Min;
  if i > Max then Result := Max;
end;


function StartLPTDriver: Boolean;
begin
   LPTPort.OpenDriver();
   if (not LPTPort.ActiveHW) then
   begin
      MessageDlg('Could not open the DriverLINX driver.', mtError, [mbOK], 0);
      Halt(0);
   end;
   Result := True;
end;

procedure WaitDisplayReady(Port: Integer; BiDi: Boolean);
// Warten, bis das Display bereit ist
var
  Tmp: Byte;
  t: TDateTime;
begin
  if BiDi then
  begin
       // LPT im ECP-Control-Register auf PS/2-Mode setzen
       // ntig, falls es ein ECP-Port ist
       Tmp := LPTPort.Port[Port + $402];
       Tmp := tmp AND $1F;
       Tmp := tmp OR $20; // Bidirektionaler Modus = PS/2
       LPTPort.Port[Port + $402] := tmp;
       t := now + (5/86400);  // Begrenzt auf 5 Sekunden
       repeat
             // Auf Eingang schalten und abfragen
             LPTPort.Port[Port + 2] := $20 + 4 + 2;     //In, CD=1, CS=0, Wr=1
             LPTPort.Port[Port + 2] := $20 + 4 + 2 + 8; //In, CD=1, CS=0, Wr=1, Rd=0
             Tmp := LPTPort.Port[Port];
             LPTPort.Port[Port + 2] := $20 + 4 + 2;     //In, CD=1, CS=0, Wr=1
       until ((Tmp AND 3) = 3) OR (now > t); // Maximal 5 Sekunden warten
       LPTPort.Port[Port + 2] := 0;
  end
  else
     //  Delay(1); // Ohne bidirektionalen Parallelport ist Handshaking nicht mglich
end;

procedure WriteData(Port: Integer; BiDi: Boolean; Data: Byte);
// Daten ins Datenregister des Displays schreiben
begin
  WaitDisplayReady(Port, BiDi);
  LPTPort.Port[Port] := Data;
  LPTPort.Port[Port + 2] := 0; (* C/D = L (Data) *)
  LPTPort.Port[Port + 2] := 2; (* CE *)
  LPTPort.Port[Port + 2] := 3; (* CE+WR *)
  LPTPort.Port[Port + 2] := 2; (* CE *)
  LPTPort.Port[Port + 2] := 0;
end;

procedure WriteCtrl(Port: Integer; BiDi: Boolean; Command: Byte);
// Daten ins Steuerregister des Displays schreiben
begin
  WaitDisplayReady(Port, BiDi);
  LPTPort.Port[Port] := Command;
  LPTPort.Port[Port + 2] := 4; (* C/D = H (ctrl) *)
  LPTPort.Port[Port + 2] := 4+2; (* CE *)
  LPTPort.Port[Port + 2] := 4+3; (* CE+WR *)
  LPTPort.Port[Port + 2] := 4+2; (* CE *)
  LPTPort.Port[Port + 2] := 0;
end;

procedure delay(ms: integer);
var t: TDateTime;
    z: integer;
begin
  t:=now+(5/86400);  // Begrenzt auf 5 Sekunden
  for z:=1 to ms do begin
    delz:=0;
    while (now<t) AND (delz<msdelay) do
      inc(delz);
  end;
end;


procedure InitLCD(Port: Integer; BiDi: Boolean);
begin
  // Display init
  WriteData(Port, BiDi, TEXT_HOME_ADDRESS and $00FF); //D1: LowAddress
  WriteData(Port, BiDi, TEXT_HOME_ADDRESS shr 8);     //D2: HighAddress
  WriteCtrl(Port, BiDi, $40);                         //Code: $40 -> 01000000

  WriteData(Port, BiDi, TEXT_COLUMS);                 //D1: Colums
  WriteData(Port, BiDi, 0);                           //D2: 0
  WriteCtrl(Port, BiDi, $41);                         //Code: $41 -> 01000001

  WriteData(Port, BiDi, GRAPH_HOME_ADDRESS and $00FF);//D1: LowAddress
  WriteData(Port, BiDi, GRAPH_HOME_ADDRESS shr 8);    //D2: HighAddress
  WriteCtrl(Port, BiDi, $42);                         //Code: $42 -> 01000010

  WriteData(Port, BiDi, GRAPH_COLUMS);                //D1: Colums
  WriteData(Port, BiDi, 0);                           //D2: 0
  WriteCtrl(Port, BiDi, $43);                         //Code: $43 -> 01000011

  WriteData(Port, BiDi, 2{CGRAM_HOME_ADDRESS and $00FF}); //D1: LowAddress
  WriteData(Port, BiDi, 0{CGRAM_HOME_ADDRESS shr 8}); //D2: HighAddress
  WriteCtrl(Port, BiDi, $22);                         //Code: $22 -> 00100010


  //Internal CGROM Mode, OR Mode
  WriteCtrl(Port, BiDi, $80); (* OR Mode *)           //80->10000000

  //DisplayMode CursorOff, BlinkOff, TextOff, GraficOff
  WriteCtrl(Port, BiDi, $90);                         //9f->10011111
end;

procedure SetAddressPointer(Port: Integer; BiDi: Boolean; Addr: Integer);
begin
  WriteData(Port, BiDi, Addr and $00FF);               //D1: LowAddress
  WriteData(Port, BiDi, Addr shr 8);                 //D2: HighAddress
  WriteCtrl(Port, BiDi, $24);                        //Code: $24 -> 00100100
end;

function ANSI2DisplayText(chr: Integer): Integer;
// Windows-Zeichensatz in Display-Zeichensatz konvertieren
//Display-Schriftsatz ist um 32 verschoben  #32 -> #0
begin
  case Chr of
       32..127  : result := Chr - 32;   //internen Schriftsatz fngt bei #32 an
       Ord('') : result := 96;
       Ord('') : result := 97;
       Ord('') : result := 98;
       Ord('') : result := 99;
       Ord('') : result := 100;
       Ord('') : result := 101;
       Ord('') : result := 102;
       Ord('') : result := 104;
       Ord('') : result := 105;
       Ord('') : result := 106;
       Ord('') : result := 107;
       Ord('') : result := 108;
       Ord('') : result := 109;
       Ord('') : result := 110;
       Ord('') : result := 111;
       Ord('') : result := 112;
       Ord('') : result := 113;
       Ord('') : result := 114;
       Ord('') : result := 115;
       Ord('') : result := 116;
       Ord('') : result := 117;
       Ord('') : result := 118;
       Ord('') : result := 119;
       Ord('') : result := 120;
       Ord('') : result := 121;
       Ord('') : result := 122;
       Ord('') : result := 123;
       Ord('') : result := 124;
       Ord('') : result := 125;
       Ord('') : result := 127;
  else
            result := 0;
  end;
end;

begin
  // Schleifenlaufzeit ausmessen, Durchlufe in einer halben Sekunde
  // brachial, funktioniert aber
  // Gedacht um nacher die Funktion delay zu ermglichen
  // eleganter ber gettickcount
  t := now + (1/(86400*2));
  msdelay:=0;
  delz:=$7FFFFFFFFFFFFFFF;
  while (now<t) AND (msdelay<delz) do
        inc(msdelay);
  msdelay:=msdelay div 500;


  LPTPort := TDLPortIO.Create(nil);
  LPTPort.DriverPath:=ExtractFileDir(ParamStr(0));
end.
