unit PollinLCDPreview;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, PollinLCD, exfont;

type
  TPollinLCDPreview = class(TPaintBox)
  private
    _PollinLCD: TPollinLCD;
    _TextData: TTextData;
    _GraphData: TGraphData;
    _FontData: TFontData;
    _Enabled: Boolean;
    procedure SetLCD(const Value: TPollinLCD);
    procedure OnTextRefresh(Owner: TObject; TextData: array of Byte);
    procedure OnGraphRefresh(Owner: TObject; GraphData: array of Byte);
    procedure OnFontRefresh(Owner: TObject; FontData: array of Byte);
    procedure Paint; override;
    procedure SetEnabled(const Value: Boolean);
  protected
    { Protected-Deklarationen }
  public
    constructor Create(AOwner: TComponent);   override;
    procedure DrawPreview;
  published
    property LCD: TPollinLCD read _PollinLCD write SetLCD;
    property Enabled: Boolean read _Enabled write SetEnabled;
  end;

implementation


{ TPollinLCDPreview }

constructor TPollinLCDPreview.Create(AOwner: TComponent);
begin
  inherited;
  _PollinLCD := TPollinLCD.Create(nil);
end;

procedure TPollinLCDPreview.OnGraphRefresh(Owner: TObject;
  GraphData: array of Byte);
var i: Integer;
begin
  for i := 1 to High(_GraphData) do
        _GraphData[i] := GraphData[i-1];
  DrawPreview;
end;

procedure TPollinLCDPreview.OnTextRefresh(Owner: TObject;
  TextData: array of Byte);
var i: Integer;
begin
  for i := 1 to High(_TextData) do
        _TextData[i] := TextData[i-1];
  DrawPreview;
end;

procedure TPollinLCDPreview.OnFontRefresh(Owner: TObject;
  FontData: array of Byte);
var i: Integer;
begin
  for i := 1 to High(_FontData) do
        _FontData[i] := FontData[i-1];
  DrawPreview;
end;


procedure TPollinLCDPreview.SetLCD(const Value: TPollinLCD);
begin
  _PollinLCD := Value;
  _PollinLCD.OnTextRefresh := OnTextRefresh;
  _PollinLCD.OnGraphRefresh := OnGraphRefresh;
  _PollinLCD.OnFontRefresh := OnFontRefresh;
end;

procedure TPollinLCDPreview.DrawPreview;
var
  x, y, i, j, k, l: Integer;
  b : byte;
  bmp: TBitmap;
begin
  if not _Enabled then Exit;
  bmp := TBitmap.Create;
  bmp.Width := 240;
  bmp.Height := 64;
  with Bmp.Canvas do
  begin
      Brush.Color := RGB(230, 240, 250);
      Pen.Color := RGB(230, 240, 250);
      Rectangle(0, 0, 240, 64);
      Pen.Color := RGB(30, 90, 140);

      for x := 1 to 40 do
      for y := 1 to 8 do
      for k := 1 to 7 do
      for l := 1 to 5 do
          if CharDOS5x7[_TextData[(y-1)*40 + x], k, l] = 1  then
               Pixels[(x - 1) * 6 + l, (y- 1) * 8 + k - 1] := Pen.Color;

      for y := 1 to 64 do
      for x := 1 to 40 do
      begin
         b := _GraphData[(y-1)*40 + x];
         for k := 5 downto 0 do
         begin
             if b mod 2 = 1 then
                 Pixels[(x - 1) * 6 + k, y- 1] := Pen.Color;
             b := b shr 1;
         end;
      end;
  end;
  self.Canvas.StretchDraw(Rect(0,0, self.Width, self.Height), Bmp);
  Bmp.Free;
end;

procedure TPollinLCDPreview.Paint;
begin
  inherited;
  self.Canvas.Brush.Color := RGB(230, 240, 250);
  self.Canvas.Pen.Color := RGB(230, 240, 250);
  self.Canvas.Rectangle(0, 0, self.Width, self.Height);
end;

procedure TPollinLCDPreview.SetEnabled(const Value: Boolean);
begin
 _Enabled := Value;
end;

end.
